"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.verifyConditons = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var app_root_path_1 = __importDefault(require("app-root-path"));
var paths_1 = require("./paths");
var errors_1 = require("./errors");
var strategies_1 = require("./strategies");
var verifyAndroidPath = function (androidPath) {
    var absAndroidPath = (0, paths_1.toAbsolutePath)(androidPath);
    if (!absAndroidPath.endsWith('build.gradle')) {
        return (0, errors_1.getSemanticReleaseError)('androidPath', 'ENRNANDROIDPATH');
    }
    if (!fs_1["default"].existsSync(absAndroidPath)) {
        return (0, errors_1.getSemanticReleaseError)('androidPath', 'ENRNANDROIDPATH');
    }
    return null;
};
var verifyIosPath = function (iosPath) {
    var absIosPath = (0, paths_1.toAbsolutePath)(iosPath);
    if (!fs_1["default"].existsSync(absIosPath)) {
        return (0, errors_1.getSemanticReleaseError)('iosPath', 'ENRNIOSPATH');
    }
    if (!fs_1["default"].lstatSync(absIosPath).isDirectory()) {
        return (0, errors_1.getSemanticReleaseError)('iosPath', 'ENRNIOSPATH');
    }
    return null;
};
var verifyJSONFile = function (fileName) {
    var filePath = path_1["default"].join(app_root_path_1["default"].path, fileName);
    if (!fs_1["default"].existsSync(filePath)) {
        return null;
    }
    try {
        JSON.parse(fs_1["default"].readFileSync(filePath, 'utf8'));
    }
    catch (error) {
        return (0, errors_1.getSemanticReleaseError)('fromFile', 'ENRNFROMFILENOTJSON');
    }
    return null;
};
var verifyConditons = function (pluginConfig) {
    var _a;
    var errors = [];
    if (pluginConfig.androidPath) {
        errors.push(verifyAndroidPath(pluginConfig.androidPath));
    }
    if (pluginConfig.iosPath) {
        errors.push(verifyIosPath(pluginConfig.iosPath));
    }
    if (pluginConfig.fromFile) {
        errors.push(verifyJSONFile(pluginConfig.fromFile));
    }
    var booleanValues = [
        'skipBuildNumber',
        'skipAndroid',
        'skipIos',
        'noPrerelease',
    ];
    booleanValues.forEach(function (key) {
        if (key in pluginConfig && typeof pluginConfig[key] !== 'boolean') {
            errors.push((0, errors_1.getSemanticReleaseError)(key, 'ENRNNOTBOOLEAN'));
        }
    });
    if ('iosPackageName' in pluginConfig
        && !(typeof pluginConfig.iosPackageName === 'string' || pluginConfig.iosPackageName instanceof String)) {
        errors.push((0, errors_1.getSemanticReleaseError)('iosPackageName', 'ENRNNOTSTRING'));
    }
    var _b = (_a = pluginConfig.versionStrategy) !== null && _a !== void 0 ? _a : {}, ios = _b.ios, android = _b.android;
    if ((ios === null || ios === void 0 ? void 0 : ios.buildNumber) && !strategies_1.iosVesionStrategies.includes(ios.buildNumber)) {
        errors.push((0, errors_1.getSemanticReleaseError)('versionStrategy', 'ENRNVERSIONSTRATEGY'));
    }
    if ((android === null || android === void 0 ? void 0 : android.buildNumber) && !strategies_1.iosVesionStrategies.includes(android.buildNumber)) {
        errors.push((0, errors_1.getSemanticReleaseError)('versionStrategy', 'ENRNVERSIONSTRATEGY'));
    }
    if ((ios === null || ios === void 0 ? void 0 : ios.preRelease) != null && typeof ios.preRelease !== 'boolean') {
        errors.push((0, errors_1.getSemanticReleaseError)('versionStrategy', 'ENRNVERSIONSTRATEGY'));
    }
    if ((android === null || android === void 0 ? void 0 : android.preRelease) != null && typeof android.preRelease !== 'boolean') {
        errors.push((0, errors_1.getSemanticReleaseError)('versionStrategy', 'ENRNVERSIONSTRATEGY'));
    }
    return errors.filter(function (x) { return x; });
};
exports.verifyConditons = verifyConditons;
//# sourceMappingURL=verify.js.map