"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.versionAndroid = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var paths_1 = require("../paths");
var utils_1 = require("./utils");
var getAndroidPath = function (androidPath) {
    var defaultAndroidPath = path_1["default"].join('android', 'app', 'build.gradle');
    return (0, paths_1.toAbsolutePath)(androidPath !== null && androidPath !== void 0 ? androidPath : defaultAndroidPath);
};
var getNextAndroidVersionCode = function (strategy, logger, version, currentVersionCode, minSdkVersion) {
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'none') {
        return currentVersionCode;
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'env') {
        var envVersionCode = process.env.ANDROID_BUILD_NUMBER;
        if (!envVersionCode) {
            logger.warn('Could not update Android versionCode using the env strategy '
                + 'as the ANDROID_BUILD_NUMBER could not be determined.');
            return currentVersionCode;
        }
        return envVersionCode;
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'relative') {
        var semanticBuildNumber = (0, utils_1.getSemanticBuildNumber)(version, logger, 'Android');
        if (!semanticBuildNumber) {
            return currentVersionCode;
        }
        return parseInt(semanticBuildNumber, 10).toString();
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'relative-extended') {
        if (!minSdkVersion) {
            logger.warn('Could not update Android versionCode using the relative-extended strategy '
                + 'as the minSdkVersion could not be determined.');
            return currentVersionCode;
        }
        if (minSdkVersion.length > 2) {
            logger.warn('Could not update Android versionCode using the relative-extended strategy '
                + 'as the minSdkVersion is greater than 99. Welcome to the future. Have the '
                + 'robots taken over yet?');
            return currentVersionCode;
        }
        var semanticBuildNumber = (0, utils_1.getSemanticBuildNumber)(version, logger, 'Android');
        if (!semanticBuildNumber) {
            return currentVersionCode;
        }
        return "".concat(minSdkVersion.padStart(2, '0'), "0").concat(semanticBuildNumber);
    }
    return String(parseInt(currentVersionCode, 10) + 1);
};
var versionFromFile = function (_a, _b, version) {
    var _c;
    var versionStrategy = _a.versionStrategy, fromFile = _a.fromFile, skipBuildNumber = _a.skipBuildNumber;
    var logger = _b.logger;
    if (skipBuildNumber) {
        logger.info('Skipping update of Android build number');
        return;
    }
    var versionFile = (0, utils_1.loadBuildVersionFile)(fromFile);
    var nextBuildVersion = getNextAndroidVersionCode(versionStrategy.android, logger, version, (_c = versionFile.android) !== null && _c !== void 0 ? _c : '0');
    (0, utils_1.writeBuildVersionFile)(fromFile, __assign(__assign({}, versionFile), { android: nextBuildVersion }));
    logger.success("Android build number > ".concat(nextBuildVersion));
};
var versionAndroid = function (pluginConfig, context) {
    var _a;
    var logger = context.logger, nextRelease = context.nextRelease;
    var version = (nextRelease !== null && nextRelease !== void 0 ? nextRelease : {}).version;
    if (!version) {
        return;
    }
    if ((0, utils_1.isPreRelease)(nextRelease)
        && ((_a = pluginConfig.versionStrategy.android) === null || _a === void 0 ? void 0 : _a.preRelease) === false) {
        logger.info('Skipping pre-release version for Android');
        return;
    }
    if (pluginConfig.fromFile) {
        logger.info("Versioning Android from file: ".concat(pluginConfig.fromFile));
        versionFromFile(pluginConfig, context, version);
        return;
    }
    logger.info('Versioning Android');
    var androidPath = getAndroidPath(pluginConfig.androidPath);
    if (!fs_1["default"].existsSync(androidPath)) {
        logger.error("No file found at ".concat(androidPath));
        return;
    }
    var gradleFile = fs_1["default"].readFileSync(androidPath).toString();
    var newBuildNumber;
    gradleFile = gradleFile.replace(/versionName (["'])(.*)["']/, "versionName $1".concat(version, "$1"));
    logger.success("Android versionName > ".concat(version));
    if (!pluginConfig.skipBuildNumber) {
        var _b = gradleFile.match(/minSdkVersion (\d+)/) || [], minSdkVersion_1 = _b[1];
        gradleFile = gradleFile.replace(/versionCode (\d+)/, function (_match, currentVersionCode) {
            newBuildNumber = getNextAndroidVersionCode(pluginConfig.versionStrategy.android, logger, version, currentVersionCode, minSdkVersion_1);
            return "versionCode ".concat(newBuildNumber);
        });
        logger.success("Android versionCode > ".concat(newBuildNumber));
    }
    fs_1["default"].writeFileSync(androidPath, gradleFile);
};
exports.versionAndroid = versionAndroid;
//# sourceMappingURL=android.js.map