"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.versionIos = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var plist_1 = __importDefault(require("plist"));
var js_beautify_1 = __importDefault(require("js-beautify"));
var xcode_1 = require("pbxproj-dom/xcode");
var lodash_uniq_1 = __importDefault(require("lodash.uniq"));
var detect_indent_1 = __importDefault(require("detect-indent"));
var lodash_flattendeep_1 = __importDefault(require("lodash.flattendeep"));
var html_minifier_1 = __importDefault(require("html-minifier"));
var paths_1 = require("../paths");
var utils_1 = require("./utils");
var getIosPath = function (iosPath) {
    var defaultIosPath = path_1["default"].join('ios');
    return (0, paths_1.toAbsolutePath)(iosPath !== null && iosPath !== void 0 ? iosPath : defaultIosPath);
};
var getIosStrictBundleVersion = function (currentBundleVersion, version) {
    var _a;
    var _b = currentBundleVersion.split('.'), majorStr = _b[0], minorStr = _b[1], patchStr = _b[2];
    var major = parseInt(majorStr !== null && majorStr !== void 0 ? majorStr : 0, 10);
    var minor = parseInt(minorStr !== null && minorStr !== void 0 ? minorStr : 0, 10);
    var patch = parseInt(patchStr !== null && patchStr !== void 0 ? patchStr : 0, 10);
    if (!currentBundleVersion) {
        return '1.1.1';
    }
    var versioned = false;
    if (patch >= 99) {
        minor += 1;
        patch = 1;
        versioned = true;
    }
    if (minor >= 99) {
        major += 1;
        minor = 1;
        patch = 1;
        versioned = true;
    }
    if (!major) {
        major += 1;
        versioned = true;
    }
    if (!minor) {
        minor += 1;
        versioned = true;
    }
    if (!versioned || !patch) {
        patch += 1;
    }
    var _c = version.split('-'), preReleaseLabel = _c[1];
    if (!preReleaseLabel) {
        return "".concat(major, ".").concat(minor, ".").concat(patch);
    }
    var preReleaseVersion = parseInt((_a = preReleaseLabel.split('.')[1]) !== null && _a !== void 0 ? _a : 1, 10);
    return "".concat(major, ".").concat(minor, ".").concat((patch * 10000) + preReleaseVersion);
};
var getIosBundleVersion = function (strategy, logger, currentBundleVersion, version) {
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'none') {
        return currentBundleVersion;
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'env') {
        var envBundleVersion = process.env.IOS_BUILD_NUMBER;
        if (!envBundleVersion) {
            logger.warn('Could not update iOS bundle version using the env strategy '
                + 'as the IOS_BUILD_NUMBER could not be determined.');
            return currentBundleVersion;
        }
        return envBundleVersion;
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'semantic') {
        return (0, utils_1.stripPrereleaseVersion)(version);
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'relative') {
        var semanticBuildNumber = (0, utils_1.getSemanticBuildNumber)(version, logger, 'iOS');
        if (!semanticBuildNumber) {
            return currentBundleVersion;
        }
        return semanticBuildNumber;
    }
    if ((strategy === null || strategy === void 0 ? void 0 : strategy.buildNumber) === 'increment') {
        var major = currentBundleVersion ? parseInt(currentBundleVersion, 10) : 0;
        return String(major + 1);
    }
    return getIosStrictBundleVersion(currentBundleVersion, version);
};
var getPlistFilenames = function (xcode) { return (0, lodash_uniq_1["default"])((0, lodash_flattendeep_1["default"])(xcode.document.projects.map(function (project) { return (project.targets.filter(Boolean).map(function (target) { return (target.buildConfigurationsList.buildConfigurations.map(function (config) {
    var _a;
    return ((_a = config.ast.value.get('buildSettings').get('INFOPLIST_FILE')) === null || _a === void 0 ? void 0 : _a.text);
})); })); }))).filter(function (x) { return x; }); };
var incrementPbxProjectBuildNumbers = function (xcode, logger, version, pluginConfig) {
    var currentProjectVersionKey = 'CURRENT_PROJECT_VERSION';
    var marketingVersionKey = 'MARKETING_VERSION';
    var iosPackageName = pluginConfig.iosPackageName, skipBuildNumber = pluginConfig.skipBuildNumber;
    xcode.document.projects.forEach(function (project) {
        var validTargets = project.targets.filter(Boolean);
        validTargets.forEach(function (target) {
            var validBuildConfigs = target.buildConfigurationsList.buildConfigurations.filter(function () { return (!iosPackageName || iosPackageName === target.name); });
            validBuildConfigs.forEach(function (buildConfig) {
                var _a, _b;
                var _c, _d;
                var currentProjectVersion = (_c = buildConfig.ast.value
                    .get('buildSettings')
                    .get(currentProjectVersionKey)) === null || _c === void 0 ? void 0 : _c.text;
                var currentMarketingVersion = (_d = buildConfig.ast.value
                    .get('buildSettings')
                    .get(marketingVersionKey)) === null || _d === void 0 ? void 0 : _d.text;
                var buildSettings = {};
                if (currentMarketingVersion) {
                    var newMarketingVersion = (0, utils_1.stripPrereleaseVersion)(version);
                    Object.assign(buildSettings, (_a = {},
                        _a[marketingVersionKey] = newMarketingVersion,
                        _a));
                    logger.success("iOS ".concat(target.name, " ").concat(marketingVersionKey, " > ").concat(newMarketingVersion));
                }
                if (currentProjectVersion && !skipBuildNumber) {
                    var newProjectVersion = getIosBundleVersion(pluginConfig.versionStrategy.ios, logger, String(currentProjectVersion), version);
                    Object.assign(buildSettings, (_b = {},
                        _b[currentProjectVersionKey] = newProjectVersion,
                        _b));
                    logger.success("iOS ".concat(target.name, " ").concat(currentProjectVersionKey, " > ").concat(newProjectVersion));
                }
                if (Object.keys(buildSettings).length) {
                    buildConfig.patch({ buildSettings: buildSettings });
                }
            });
        });
    });
    xcode.save();
    var versionRE = new RegExp("(".concat(currentProjectVersionKey, "|").concat(marketingVersionKey, ") = \"([^\"]*)\""), 'gm');
    var xcodeDoc = fs_1["default"].readFileSync(xcode.path, { encoding: 'utf-8' });
    if (!xcodeDoc) {
        throw new Error("Could not read Xcode project file at ".concat(xcode.path));
    }
    var updatedXcodeDoc = xcodeDoc.replace(versionRE, '$1 = $2');
    fs_1["default"].writeFileSync(xcode.path, updatedXcodeDoc, { encoding: 'utf8' });
};
var isPlistObject = function (value) { return (typeof value === 'object'); };
var updateCfBundleVersion = function (pluginConfig, plistFilename, plistObj, version, logger) {
    var _a;
    var key = 'CFBundleVersion';
    var currentBuildVersion = plistObj[key] ? String(plistObj[key]) : '';
    var newBuildVersion = getIosBundleVersion(pluginConfig.versionStrategy.ios, logger, currentBuildVersion, version);
    if (currentBuildVersion.startsWith('$(')) {
        logger.info("Not updating iOS ".concat(plistFilename, " ").concat(key, " as it is the variable \"").concat(currentBuildVersion, "\""));
        return;
    }
    Object.assign(plistObj, (_a = {}, _a[key] = newBuildVersion, _a));
    logger.success("iOS ".concat(plistFilename, " ").concat(key, " > ").concat(newBuildVersion));
};
var updateCfBundleShortVersion = function (plistFilename, plistObj, version, logger) {
    var _a;
    var key = 'CFBundleShortVersionString';
    var shortVersion = (0, utils_1.stripPrereleaseVersion)(version);
    var currentVersion = plistObj[key];
    if (String(currentVersion).startsWith('$(')) {
        logger.info("Not updating iOS ".concat(plistFilename, " ").concat(key, " as it is the variable \"").concat(currentVersion, "\""));
        return;
    }
    Object.assign(plistObj, (_a = {}, _a[key] = shortVersion, _a));
    logger.success("iOS ".concat(plistFilename, " ").concat(key, " > ").concat(shortVersion));
};
var resetEmptyTagStyles = function (originalContent, newContent) {
    var originalLines = originalContent.trim().split(/\r?\n/);
    var newLines = newContent.trim().split(/\r?\n/);
    if (originalLines.length !== newLines.length) {
        return newContent;
    }
    return newLines.map(function (newLine, index) {
        var originalLine = originalLines[index];
        if (html_minifier_1["default"].minify(newLine).trim() === html_minifier_1["default"].minify(originalLine).trim()) {
            return originalLine;
        }
        return newLine;
    }).join('\n');
};
var incrementPlistVersions = function (pluginConfig, xcode, iosPath, version, logger) {
    var plistFilenames = getPlistFilenames(xcode);
    plistFilenames
        .filter(function (plistFilename) { return (!pluginConfig.iosPackageName
        || path_1["default"].dirname(plistFilename) === pluginConfig.iosPackageName); })
        .forEach(function (plistFilename) {
        var _a, _b, _c, _d, _e;
        var plistPath = path_1["default"].join(iosPath, plistFilename);
        var originalPlistFile = fs_1["default"].readFileSync(plistPath).toString();
        var plistObj = plist_1["default"].parse(originalPlistFile);
        if (!isPlistObject(plistObj)) {
            return;
        }
        updateCfBundleShortVersion(plistFilename, plistObj, version, logger);
        if (!pluginConfig.skipBuildNumber) {
            updateCfBundleVersion(pluginConfig, plistFilename, plistObj, version, logger);
        }
        var indent = (0, detect_indent_1["default"])(originalPlistFile);
        var dictPattern = /<dict>[\s\S]*<\/dict>/i;
        var newPlistFile = plist_1["default"].build(plistObj);
        var newPlistContent = (_b = (_a = newPlistFile.match(dictPattern)) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : '';
        var xmlDeclaration = ((_c = originalPlistFile.match(/<\?xml.*\?>/i)) !== null && _c !== void 0 ? _c : [])[0];
        var doctypeElement = ((_d = originalPlistFile.match(/<\?doctype.*\?>/i)) !== null && _d !== void 0 ? _d : [])[0];
        var plistElement = ((_e = originalPlistFile.match(/<\?plist.*\?>/i)) !== null && _e !== void 0 ? _e : [])[0];
        var beautifyOpts = {};
        if (indent.type === 'tab') {
            beautifyOpts.indent_with_tabs = true;
        }
        else {
            beautifyOpts.indent_size = indent.amount;
        }
        var cleanPlistFile = [
            xmlDeclaration !== null && xmlDeclaration !== void 0 ? xmlDeclaration : '<?xml version="1.0" encoding="UTF-8"?>',
            doctypeElement !== null && doctypeElement !== void 0 ? doctypeElement : '<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">',
            plistElement !== null && plistElement !== void 0 ? plistElement : '<plist version="1.0">',
            newPlistContent,
            '</plist>',
        ].join('\n');
        cleanPlistFile = cleanPlistFile.replace(dictPattern, js_beautify_1["default"].html(newPlistContent, beautifyOpts));
        fs_1["default"].writeFileSync(plistPath, "".concat(resetEmptyTagStyles(originalPlistFile, cleanPlistFile), "\n"));
    });
};
var versionFromFile = function (_a, _b, version) {
    var _c;
    var versionStrategy = _a.versionStrategy, fromFile = _a.fromFile, skipBuildNumber = _a.skipBuildNumber;
    var logger = _b.logger;
    if (skipBuildNumber) {
        logger.info('Skipping update of iOS Android build number');
        return;
    }
    var versionFile = (0, utils_1.loadBuildVersionFile)(fromFile);
    var nextBuildVersion = getIosBundleVersion(versionStrategy.ios, logger, (_c = versionFile.ios) !== null && _c !== void 0 ? _c : '0', version);
    (0, utils_1.writeBuildVersionFile)(fromFile, __assign(__assign({}, versionFile), { ios: nextBuildVersion }));
    logger.success("iOS build number > ".concat(nextBuildVersion));
};
var versionIos = function (pluginConfig, context) {
    var _a;
    var logger = context.logger, nextRelease = context.nextRelease;
    var version = (nextRelease !== null && nextRelease !== void 0 ? nextRelease : {}).version;
    if (!version) {
        return;
    }
    var _b = (_a = pluginConfig.versionStrategy.ios) !== null && _a !== void 0 ? _a : {}, preRelease = _b.preRelease, buildNumber = _b.buildNumber;
    if ((0, utils_1.isPreRelease)(nextRelease)
        && (preRelease === false
            || (buildNumber
                && buildNumber !== 'strict'
                && buildNumber !== 'env'))) {
        logger.info('Skipping pre-release version for iOS');
        return;
    }
    if (pluginConfig.fromFile) {
        logger.info("Versioning iOS from file: ".concat(pluginConfig.fromFile));
        versionFromFile(pluginConfig, context, version);
        return;
    }
    logger.info('Versioning iOS');
    var iosPath = getIosPath(pluginConfig.iosPath);
    var xcodeProjects = fs_1["default"]
        .readdirSync(iosPath)
        .filter(function (file) { return /\.xcodeproj$/i.test(file); });
    if (!xcodeProjects.length) {
        logger.error("No Xcode project file found at ".concat(iosPath));
        return;
    }
    var projectFolder = path_1["default"].join(iosPath, xcodeProjects[0]);
    var xcode = xcode_1.Xcode.open(path_1["default"].join(projectFolder, 'project.pbxproj'));
    incrementPbxProjectBuildNumbers(xcode, logger, version, pluginConfig);
    incrementPlistVersions(pluginConfig, xcode, iosPath, version, logger);
};
exports.versionIos = versionIos;
//# sourceMappingURL=ios.js.map