"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.writeBuildVersionFile = exports.loadBuildVersionFile = exports.isPreRelease = exports.getSemanticBuildNumber = exports.stripPrereleaseVersion = void 0;
var semver_1 = __importDefault(require("semver"));
var fs_1 = __importDefault(require("fs"));
var os_1 = require("os");
var path_1 = __importDefault(require("path"));
var app_root_path_1 = __importDefault(require("app-root-path"));
var errors_1 = require("../errors");
var stripPrereleaseVersion = function (version) {
    var major = semver_1["default"].major(version);
    var minor = semver_1["default"].minor(version);
    var patch = semver_1["default"].patch(version);
    return "".concat(major, ".").concat(minor, ".").concat(patch);
};
exports.stripPrereleaseVersion = stripPrereleaseVersion;
var getSemanticBuildNumber = function (version, logger, platform) {
    var major = String(semver_1["default"].major(version)).padStart(2, '0');
    var minor = String(semver_1["default"].minor(version)).padStart(2, '0');
    var patch = String(semver_1["default"].patch(version)).padStart(2, '0');
    var semanticBuildNumber = "".concat(major).concat(minor).concat(patch);
    if (major.length > 2 || minor.length > 2 || patch.length > 2) {
        logger.warn("Could not update ".concat(platform, " bundle version using the relative strategy ")
            + 'as the numbers in your semantic version exceed two digits. It is '
            + 'recommended that you switch to the increment strategy (see plugin docs).');
        return null;
    }
    return semanticBuildNumber;
};
exports.getSemanticBuildNumber = getSemanticBuildNumber;
var isPreRelease = function (nextRelease) {
    var version = (nextRelease !== null && nextRelease !== void 0 ? nextRelease : {}).version;
    return version && semver_1["default"].prerelease(version);
};
exports.isPreRelease = isPreRelease;
var loadBuildVersionFile = function (fileName) {
    var filePath = path_1["default"].join(app_root_path_1["default"].path, fileName);
    if (!fs_1["default"].existsSync(filePath)) {
        return {};
    }
    var fileContent = fs_1["default"].readFileSync(filePath, 'utf8');
    var parsedContent;
    try {
        parsedContent = JSON.parse(fileContent);
    }
    catch (err) {
        throw new Error("Could not parse ".concat(fileName, ": ").concat((0, errors_1.toError)(err).message));
    }
    return parsedContent;
};
exports.loadBuildVersionFile = loadBuildVersionFile;
var writeBuildVersionFile = function (fileName, newVersionFile) {
    var filePath = path_1["default"].join(app_root_path_1["default"].path, fileName);
    var fileContent = JSON.stringify(newVersionFile, null, 2);
    fs_1["default"].writeFileSync(filePath, fileContent, 'utf8');
    fs_1["default"].appendFileSync(filePath, os_1.EOL, 'utf8');
};
exports.writeBuildVersionFile = writeBuildVersionFile;
//# sourceMappingURL=utils.js.map